const STORAGE_KEY = {
    lang: "LANG"
}

function triggerApi(type, data,cb) {
    window.electronAPI.triggerApi(type, data).then((res) => {
        cb && cb(null,res)
    }).catch((e) => {
        alert(e)
        cb && cb(e)
    })
}

const messages = {
    en: {
        dirTitle: "Select storage folder",
        dirDesc: "Please confirm whether the disk has enough free space according to your backup requirements",
        serverStatus: 'Service Status:',
        serverStatus_init: "waiting for initialization",
        serverStatus_running: "Running",
        lang: 'Language',
        changePath: 'Modify location',
        viewFolder: 'Open Folder',
        fileDir: 'photo backup folder',
        fileDirDesc: 'used to store pictures and videos backed up by App and webpage',
        fileDirDesc2: 'Reminder: After this folder is confirmed, it cannot be changed; when App backup or web page upload, if you need to backup to another directory, you can modify the backup destination',
        cacheDir: 'cache folder',
        cacheDirDesc: 'The folder used to store thumbnails and other files',
        cacheDirDesc2: 'Reference: 5000 photos + 1000 videos, a cache file of 1GB is expected to be generated',
        save: 'OK',
        refreshStatus: 'Refresh status',
        showWeb: 'Open web page',
        pcIp: 'Local IP:',
        serverPort: 'Port number:',
        serverLog: 'View log files',
        initLoadingDesc: 'Starting...',
        status_running: 'Running',
        tools: 'Tools:',
        pgRestore: 'Import database file',
        restartServer: 'Restart service',
        pgRestoreSuccess: "Import database backup successfully",
        resetAdminPwd: "Reset administrator password",
        resetAdminPwdSuccess: "Admin password has been reset, username: {username} password: {password}, password has been automatically copied",
        serverWebDesc: "Prompt: When logging in to the App, please fill in {ip}:{serverPort} for the server address; to access the webpage, please click the [Open Webpage] button; if you cannot access, please try other IPs in [Local IP].",
        loading_check: "Initialization",
        loading_fail: "Initialization failed, please check the log file",
        loading_db_init: "Database initialization",
        loading_redis_start: "Component preparation",
        loading_db_start: "Database starting",
        loading_server_start: "Web service starting",
        loading_running: "Running",
        hasNonASCIIDesc: "The current path contains special characters",
        hasNonASCIIDesc2: "Please close the program, then move the folder to the plain English path and run it again",
        hasNonASCIIDesc3: "For example: C:\\Program Files\\mt-photos-server\\",
        resetAdminPwdErr: "Administrator password reset failed, please check the log file for error information",
        systemInfo:"System Info",
        tfInfo:"Debug Info",
        quit:"Quit",
    },
    zh: {
        dirTitle:"选择存储文件夹",
        dirDesc:"请根据您的备份需求，确认所在磁盘是否有足够的剩余空间",
        serverStatus: '服务状态：',
        serverStatus_init: "等待初始化",
        serverStatus_running: "运行中",
        lang: '语言',
        changePath: '修改位置',
        viewFolder: '打开文件夹',
        fileDir: '照片备份文件夹',
        fileDirDesc: '用于存放App和网页备份的图片、视频',
        fileDirDesc2: '提示：这个文件夹确定后，将无法更改；如果需要备份到其他目录，可在App备份或网页上传时修改备份目的地',
        cacheDir: '缓存文件夹',
        cacheDirDesc: '用于存放缩略图等文件的文件夹',
        cacheDirDesc2: '参考：5000个照片+1000个视频，预计会生成1GB左右的缓存文件',
        save: '确定',
        refreshStatus: '刷新状态',
        showWeb: '打开网页',
        pcIp: '本机IP：',
        serverPort: '端口号：',
        serverLog: '查看日志文件',
        initLoadingDesc: '启动中...',
        status_running: '运行中',
        tools: '工具：',
        pgRestore: '导入数据库文件',
        restartServer: '重启服务',
        pgRestoreSuccess:"导入数据库备份成功",
        resetAdminPwd:"重置管理员密码",
        envConfig:"环境变量配置",
        resetAdminPwdSuccess:"管理密码已重置，用户名：{username} 密码：{password} ，密码已自动复制",
        serverWebDesc:"提示：App登录时，服务端地址请填写 {ip}:{serverPort}；网页访问请点击【打开网页】按钮；如果{ip}无法访问，请使用本机其他IP地址加端口号访问",
        loading_check:"初始化",
        loading_fail: "初始化失败，请查看日志文件",
        loading_db_init:"数据库初始化中",
        loading_redis_start:"组件准备中",
        loading_db_start:"数据库启动中",
        loading_server_start:"Web服务启动中",
        loading_running:"运行中",
        hasNonASCIIDesc:"当前路径包含中文或特殊字符",
        hasNonASCIIDesc2:"请关闭程序，然后移动文件夹到纯英文路径中再运行",
        hasNonASCIIDesc3:"比如： C:\\Program Files\\mt-photos-server\\",
        resetAdminPwdErr:"管理员密码重置失败，错误信息请查看日志文件",
        systemInfo:"系统信息",
        tfInfo:"调试信息",
        quit:"退出",
    }
}

// https://kazupon.github.io/vue-i18n/
const i18n = new VueI18n({
    locale: 'zh',
    messages,
})

//https://v2.cn.vuejs.org/v2/guide/events.html
const app = new Vue({
    i18n,
    el: '#app',
    data: {
        dbStatus: "",
        redisStatus: "",
        serverStatus: "",
        pgPort: 28032,
        redisPort: 28079,
        serverPort: 8063,
        cacheDir: "",
        fileDir: "",
        ipList: [],
        ip: "",
        exePath:"",
        pgInRestore:false
    },
    computed: {
        startPhase:function (){
            // check | fail | db_init |  redis_start  |  db_start | server_start | running
            if(this.serverStatus==="fail"){
                return "fail"
            }
            if(this.serverStatus==="running"){
                return "running"
            }
            if(this.dbStatus==="running"){
                return "server_start"
            }
            if(this.redisStatus==="running"){
                return "db_start"
            }
            if(this.dbStatus===""){
                return "redis_start"
            }
            if(this.dbStatus==="initDb"){
                return "db_init"
            }
            return "check"
        },
        showInit: function () {
            return [
                "CACHE_DIR_PATH",
                "CACHE_DIR_ACCESS",
                "FILE_DIR_PATH",
                "FILE_DIR_ACCESS",
            ].includes(this.serverStatus)
        }
    },
    created: function () {
        document.body.className=""
        autoSetLang()
        this.refreshStatus("all")
    },
    methods: {
        pickDir: function (type) {
            let nowPath = this.fileDir
            if (type === "cacheDir") {
                nowPath = this.cacheDir
            }
            window.electronAPI.openFile(nowPath).then((res) => {
                if (res) {
                    if (type === "cacheDir") {
                        this.cacheDir=res
                    }else{
                        this.fileDir = res
                    }
                }
            }).catch((e) => {
                console.log("catch error:", e)
            })
        },
        switchLang: function () {
            if (i18n.locale === "zh") {
                i18n.locale = 'en'
            } else {
                i18n.locale = 'zh'
            }
            localStorage.setItem(STORAGE_KEY.lang, i18n.locale)
        },
        refreshStatus: function (type) {
            window.electronAPI.getServerStatus(type).then((status) => {
                for (const _key in status) {
                    if (status.hasOwnProperty(_key)) {
                        this[_key] = status[_key]
                        if (_key === "ipList" && this.ip === "" && status[_key] && status[_key].length) {
                            this.ip = status[_key][0]
                        }
                    }
                }
                if(status && status.serverStatus===""){
                    setTimeout(()=>{
                        this.refreshStatus()
                    },500)
                }
            }).catch((e) => {
                console.log("catch error:", e)
            })
        },
        save: function () {
            window.electronAPI.saveConfig({
                fileDir: this.fileDir,
                cacheDir: this.cacheDir
            }).then(() => {
                this.serverStatus=""
                setTimeout(()=>{
                    this.refreshStatus("all")
                },500)
            }).catch((e) => {
                console.log("catch error:", e)
            })

        },
        viewFolder: function (type) {
            window.electronAPI.openPath(type).then((res) => {

            }).catch((e) => {
                console.log("catch error:", e)
            })
        },
        showWeb: function () {
            this.refreshStatus()
            triggerApi('openExternal', {url: `http://${this.ip}:${this.serverPort}`})
        },
        showLog: function () {
            window.electronAPI.openPath("log").then((res) => {

            }).catch((e) => {
                console.log("catch error:", e)
            })
        },
        reload:function (){
            location.reload()
        },
        restartServer:function (){
            triggerApi("restartServer",{})
            this.serverStatus=""
            this.refreshStatus()
        },
        pgRestore:function (){
            if(this.pgInRestore){
                return
            }
            this.pgInRestore=true
            triggerApi("pgRestore",{},(err,res)=>{
                this.pgInRestore=false
                if(res==="canceled"){
                    return
                }
                alert(i18n.t("pgRestoreSuccess"))
            })
        },
        resetAdminPwd:function (){
            triggerApi("resetAdminPwd",{},(err,res)=>{
                if(res ){
                    if( res.msg){
                        alert(i18n.t("resetAdminPwdErr"))
                    }else{
                        const {username,password}=res
                        alert(i18n.t("resetAdminPwdSuccess",{ username, password}))
                    }
                }
            })
        },
        envConfig:function (){
            triggerApi("openEnvConfig",{},(err,res)=>{

            })
        },
        showSystemInfo:function (){
            triggerApi("systemInfo",{},(err,res)=>{
                if(res){
                    alert(res)
                }
            })
        },
        quit:function (){
            triggerApi("quit",{},(err,res)=>{

            })
        },
        showTfInfo:function (){
            triggerApi("debugInfo",{},(err,res)=>{

            })
        }
    }
})


function autoSetLang() {
    const localData = localStorage.getItem(STORAGE_KEY.lang)
    if (['zh', 'en'].includes(localData)) {
        i18n.locale = localData
        return
    }
    const defaultLang = getBrowserLang()
    if (defaultLang === "zh") {
        i18n.locale = 'zh'
    } else {
        i18n.locale = 'en'
    }
}

function getBrowserLang() {
    const lang = navigator.language || navigator.userLanguage;
    return lang.substr(0, 2);
}
